// HCNObject.cpp : implementation file
/////////////////////////////////////////////////////////////////////
// Author: Husni Che Ngah
// Email:  hcnhusni@streamyx.com
/////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "HitTester.h"
#include "HCNObject.h"


// CHCN_Object
IMPLEMENT_SERIAL(CHCN_Object, CObject, 0)
CHCN_Object::CHCN_Object()
{
	m_obType = HCN_None;
	m_bSelected = FALSE;
}

CHCN_Object::~CHCN_Object()
{
}

void CHCN_Object::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{	// storing code
		
	}
	else
	{	// loading code

	}
}

// CHCN_ObjectArray
IMPLEMENT_SERIAL(CHCN_ObjectArray, CObArray, 0)
CHCN_ObjectArray::CHCN_ObjectArray()
{
}

CHCN_ObjectArray::~CHCN_ObjectArray()
{
}

void CHCN_ObjectArray::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{	// storing code
		
	}
	else
	{	// loading code

	}
}

// CHCN_ObjectPoint
IMPLEMENT_SERIAL(CHCN_ObjectPoint, CHCN_Object, 0)

CHCN_ObjectPoint::CHCN_ObjectPoint()
{
	m_dwX=0;
	m_dwY=0;
	SetObjectType(HCN_None);
}

CHCN_ObjectPoint::~CHCN_ObjectPoint()
{
}

// CHCN_ObjectLine
IMPLEMENT_SERIAL(CHCN_ObjectLine, CHCN_Object, 0)
CHCN_ObjectLine::CHCN_ObjectLine()
{
	SetObjectType(HCN_Line);		// Base class attribute

	m_lineColor = Color(255,0,0,0);
	m_lineWidth = 1;
	m_bCurved = FALSE;
	m_bShowNode = FALSE;
	m_ptArray = new CHCN_ObjectArray;
}

CHCN_ObjectLine::~CHCN_ObjectLine()
{
	if(m_ptArray->GetSize() > 0)
	{
		for(int i=0; i<m_ptArray->GetSize(); i++)
		{
			delete m_ptArray->GetAt(i);
		}
		m_ptArray->RemoveAll();
		delete m_ptArray;
	}

	m_ptArray = NULL;
}

CHCN_ObjectLine::CHCN_ObjectLine(Point* points, INT_PTR nCount, Color &color, REAL rWidth)
{
	SetObjectType(HCN_Line);		// Base class attribute
	m_lineColor = color;
	m_lineWidth = rWidth;
	m_bCurved = FALSE;
	for(int i=0; i<nCount; i++)
	{
		CHCN_ObjectPoint* pt = new CHCN_ObjectPoint(points[i]);
		m_ptArray->Add(pt);
	}
}

CHCN_ObjectLine::CHCN_ObjectLine(CHCN_ObjectArray* ptArray, Color &color, REAL rWidth)
{
	SetObjectType(HCN_Line);		// Base class attribute
	m_lineColor = color;
	m_lineWidth = rWidth;
	m_bCurved = FALSE;
	m_ptArray = ptArray;		// Initialize the new point array
}

void CHCN_ObjectLine::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{	// storing code
		
	}
	else
	{	// loading code

	}
}

void CHCN_ObjectLine::DrawObject(Graphics& graphics)
{
	if(m_bCurved)
		DrawCurvedLine(graphics);
	else
		DrawNormalLine(graphics);
}

void CHCN_ObjectLine::DrawNormalLine(Graphics& graphics)
{
	if(GetPointCount() > 0)
	{
		Pen strokePen(m_lineColor, m_lineWidth);
		Pen hilitePen(Color(128,0,255,0), m_lineWidth);

		if(m_bSelected)
			graphics.DrawLines(&hilitePen, this->GetPointArray(), (unsigned int)this->GetPointCount());
		else
			graphics.DrawLines(&strokePen, this->GetPointArray(), (unsigned int)this->GetPointCount());

		if(m_bShowNode)
			DrawSegmentNode(graphics);
	}
}

void CHCN_ObjectLine::DrawCurvedLine(Graphics& graphics)
{
	if(GetPointCount() > 0)
	{
		GraphicsPath path;
		path.AddCurve(this->GetPointArray(), (unsigned int)this->GetPointCount());

		Pen strokePen(m_lineColor, m_lineWidth);
		Pen hilitePen(Color(128,0,255,0), m_lineWidth);

		if(m_bSelected)
			graphics.DrawPath(&hilitePen, &path);
		else
			graphics.DrawPath(&strokePen, &path);

		if(m_bShowNode)
			DrawSegmentNode(graphics);
	}
}

void CHCN_ObjectLine::DrawSegmentNode(Graphics& graphics)
{
	// Draw segment nodes
	for(int i=0; i<m_ptArray->GetSize(); i++)
	{
		CHCN_ObjectPoint* pt = (CHCN_ObjectPoint*)m_ptArray->GetAt(i);
		Rect rect(pt->GetGDIPoint().X-1, pt->GetGDIPoint().Y-1, 2, 2);
		graphics.FillRectangle(&SolidBrush(Color(128, 0,0,255)), rect);
		graphics.DrawRectangle(&Pen(Color(255,0,0,255)), rect);
	}
}

Point* CHCN_ObjectLine::GetPointArray() const 
{
	Point* points = new Point[m_ptArray->GetSize()];
	for(int i=0; i<m_ptArray->GetSize(); i++)
	{
		CHCN_ObjectPoint* pPt = (CHCN_ObjectPoint*)m_ptArray->GetAt(i);
		points[i] = pPt->GetGDIPoint();
	}
	return points;
};

void CHCN_ObjectLine::SetPoints(Point* points, INT_PTR nCnt)
{
	if(m_ptArray->GetSize() > 0)
	{
		m_ptArray->RemoveAll();
		m_ptArray->FreeExtra();
	}

	for(int i=0; i<nCnt; i++)
	{
		CHCN_ObjectPoint* pPt = new CHCN_ObjectPoint(points[i]);
		m_ptArray->Add(pPt); 
	}
};

BOOL CHCN_ObjectLine::IsNodeHit(CPoint& pt)
{
	VERIFY(m_ptArray->GetSize() > 0);
	BOOL bRet = FALSE;

	if(GetPointCount() > 0)
	{
		for(int i=0; i<m_ptArray->GetSize(); i++)
		{
			Rect rect(GetPointAt(i).X-2, GetPointAt(i).Y-2, 3, 3);
			CRect rc(rect.GetLeft(), rect.GetTop(), rect.GetRight(), rect.GetBottom());

			if(rc.PtInRect(pt))
			{
				bRet = TRUE;
				goto exit_loop;
			}
		}
	}
	else
		bRet = FALSE;

exit_loop:
	return bRet;
}

BOOL CHCN_ObjectLine::IsSegmentHit(CPoint& pt)
{
	BOOL bRet = FALSE;

	if(GetPointCount() > 0)
	{
		GraphicsPath path;
		Pen strokePen(m_lineColor, 3);
		if(m_bCurved)
		{
			path.AddCurve(this->GetPointArray(), (unsigned int)this->GetPointCount());
			if(path.IsOutlineVisible(pt.x, pt.y, &strokePen))
				bRet = TRUE;
		}
		else
		{
			path.AddLines(this->GetPointArray(), (unsigned int)this->GetPointCount());
			if(path.IsOutlineVisible(pt.x, pt.y, &strokePen))
				bRet = TRUE;
		}
	}
	else
		bRet = FALSE;

	return bRet;
}